@@ -183,7 +183,7 @@
 #elif defined(__INTEL_COMPILER)
 #    define OIIO_ALIGN(size) __declspec(align((size)))
 #else
-#    error "Don't know how to define OIIO_ALIGN"
+#    define OIIO_ALIGN(size) alignas(size)
 #endif
 
 // Cache line size is 64 on all modern x86 CPUs. If this changes or we
@@ -345,10 +345,12 @@
 
 // OIIO_PRETTY_FUNCTION gives a text string of the current function
 // declaration.
-#ifndef _MSC_VER
+#if defined(__PRETTY_FUNCTION__)
 #    define OIIO_PRETTY_FUNCTION __PRETTY_FUNCTION__ /* gcc, clang */
-#else
+#elif defined(__FUNCSIG__)
 #    define OIIO_PRETTY_FUNCTION __FUNCSIG__ /* MS gotta be different */
+#else
+#    define OIIO_PRETTY_FUNCTION __FUNCTION__
 #endif
 
 
