@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 #include <cmath>
 #include <cstdio>
@@ -175,14 +149,16 @@ ImageInput::read_scanline(int y, int z, TypeDesc format, void* data,
     // native_data is true if the user asking for data in the native format
     bool native_data = (format == TypeDesc::UNKNOWN
                         || (format == m_spec.format && !perchanfile));
+    // buffer_pixel_bytes is the size in the buffer
+    stride_t buffer_pixel_bytes = native_data
+                                      ? native_pixel_bytes
+                                      : format.size() * m_spec.nchannels;
     if (native_data && xstride == AutoStride)
         xstride = native_pixel_bytes;
     else
         m_spec.auto_stride(xstride, format, m_spec.nchannels);
     // Do the strides indicate that the data area is contiguous?
-    bool contiguous = (native_data && xstride == native_pixel_bytes)
-                      || (!native_data
-                          && xstride == (stride_t)m_spec.pixel_bytes(false));
+    bool contiguous = (xstride == buffer_pixel_bytes);
 
     // If user's format and strides are set up to accept the native data
     // layout, read the scanline directly into the user's buffer.
@@ -192,12 +168,13 @@ ImageInput::read_scanline(int y, int z, TypeDesc format, void* data,
 
     // Complex case -- either changing data type or stride
     int scanline_values = m_spec.width * m_spec.nchannels;
-    unsigned char* buf  = (unsigned char*)alloca(m_spec.scanline_bytes(true));
+    unsigned char* buf  = OIIO_ALLOCA(unsigned char,
+                                     m_spec.scanline_bytes(true));
     bool ok = read_native_scanline(current_subimage(), current_miplevel(), y, z,
                                    buf);
     if (!ok)
         return false;
-    if (!perchanfile) {
+    if (m_spec.channelformats.empty()) {
         // No per-channel formats -- do the conversion in one shot
         ok = contiguous ? convert_types(m_spec.format, buf, format, data,
                                         scanline_values)
@@ -275,14 +252,14 @@ ImageInput::read_scanlines(int subimage, int miplevel, int ybegin, int yend,
     stride_t zstride = AutoStride;
     spec.auto_stride(xstride, ystride, zstride, format, nchans, spec.width,
                      spec.height);
-    bool contiguous = (xstride == (stride_t)native_pixel_bytes
-                       && ystride == (stride_t)native_scanline_bytes);
-    // If user's format and strides are set up to accept the native data
-    // layout, read the scanlines directly into the user's buffer.
-    bool rightformat = (format == TypeDesc::UNKNOWN)
-                       || (format == spec.format
-                           && spec.channelformats.empty());
-    if (rightformat && contiguous) {
+    stride_t buffer_pixel_bytes = native ? native_pixel_bytes
+                                         : format.size() * nchans;
+    stride_t buffer_scanline_bytes = native ? native_scanline_bytes
+                                            : buffer_pixel_bytes * spec.width;
+    bool contiguous = (xstride == (stride_t)buffer_pixel_bytes
+                       && ystride == (stride_t)buffer_scanline_bytes);
+
+    if (native && contiguous) {
         if (chbegin == 0 && chend == spec.nchannels)
             return read_native_scanlines(subimage, miplevel, ybegin, yend, z,
                                          data);
@@ -395,11 +372,11 @@ ImageInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
     // copies the appropriate subset.
     size_t prefix_bytes   = spec.pixel_bytes(0, chbegin, true);
     size_t subset_bytes   = spec.pixel_bytes(chbegin, chend, true);
-    size_t subset_ystride = spec.width * subset_bytes;
+    size_t subset_ystride = size_t(spec.width) * subset_bytes;
 
     // Read all channels of the scanlines into a temp buffer.
     size_t native_pixel_bytes = spec.pixel_bytes(true);
-    size_t native_ystride     = spec.width * native_pixel_bytes;
+    size_t native_ystride     = size_t(spec.width) * native_pixel_bytes;
     std::unique_ptr<char[]> buf(new char[native_ystride * (yend - ybegin)]);
     yend    = std::min(yend, spec.y + spec.height);
     bool ok = read_native_scanlines(subimage, miplevel, ybegin, yend, z,
@@ -446,12 +423,14 @@ ImageInput::read_tile(int x, int y, int z, TypeDesc format, void* data,
         xstride = native_pixel_bytes;
     m_spec.auto_stride(xstride, ystride, zstride, format, m_spec.nchannels,
                        m_spec.tile_width, m_spec.tile_height);
+    stride_t buffer_pixel_bytes = native_data
+                                      ? native_pixel_bytes
+                                      : format.size() * m_spec.nchannels;
     // Do the strides indicate that the data area is contiguous?
-    bool contiguous = (native_data && xstride == native_pixel_bytes)
-                      || (!native_data
-                          && xstride == (stride_t)m_spec.pixel_bytes(false));
-    contiguous &= (ystride == xstride * m_spec.tile_width
-                   && (zstride == ystride * m_spec.tile_height || zstride == 0));
+    bool contiguous
+        = xstride == buffer_pixel_bytes
+          && (ystride == xstride * m_spec.tile_width
+              && (zstride == ystride * m_spec.tile_height || zstride == 0));
 
     // If user's format and strides are set up to accept the native data
     // layout, read the tile directly into the user's buffer.
@@ -467,7 +446,7 @@ ImageInput::read_tile(int x, int y, int z, TypeDesc format, void* data,
                                &buf[0]);
     if (!ok)
         return false;
-    if (!perchanfile) {
+    if (m_spec.channelformats.empty()) {
         // No per-channel formats -- do the conversion in one shot
         ok = contiguous ? convert_types(m_spec.format, &buf[0], format, data,
                                         tile_values)
@@ -547,11 +526,11 @@ ImageInput::read_tiles(int subimage, int miplevel, int xbegin, int xend,
     if (spec.undefined())
         return false;
 
-    chend      = clamp(chend, chbegin + 1, spec.nchannels);
-    int nchans = chend - chbegin;
+    chend = clamp(chend, chbegin + 1, spec.nchannels);
     if (!spec.valid_tile_range(xbegin, xend, ybegin, yend, zbegin, zend))
         return false;
 
+    int nchans = chend - chbegin;
     // native_pixel_bytes is the size of a pixel in the FILE, including
     // the per-channel format.
     stride_t native_pixel_bytes = (stride_t)spec.pixel_bytes(chbegin, chend,
@@ -694,9 +673,6 @@ ImageInput::read_tiles(int subimage, int miplevel, int xbegin, int xend,
         }
     }
 
-    if (!ok)
-        errorf("ImageInput::read_tiles : no support for format %s",
-               spec.format);
     return ok;
 }
 
@@ -903,7 +879,8 @@ ImageInput::read_image(int subimage, int miplevel, int chbegin, int chend,
         // we may someday want to revisit this to batch multiple rows.
         for (int z = 0; z < spec.depth; z += spec.tile_depth) {
             for (int y = 0; y < spec.height && ok; y += spec.tile_height) {
-                ok &= read_tiles(spec.x, spec.x + spec.width, y + spec.y,
+                ok &= read_tiles(subimage, miplevel, spec.x,
+                                 spec.x + spec.width, y + spec.y,
                                  std::min(y + spec.y + spec.tile_height,
                                           spec.y + spec.height),
                                  z + spec.z,
@@ -928,8 +905,8 @@ ImageInput::read_image(int subimage, int miplevel, int chbegin, int chend,
         for (int z = 0; z < spec.depth; ++z) {
             for (int y = 0; y < spec.height && ok; y += chunk) {
                 int yend = std::min(y + spec.y + chunk, spec.y + spec.height);
-                ok &= read_scanlines(y + spec.y, yend, z + spec.z, chbegin,
-                                     chend, format,
+                ok &= read_scanlines(subimage, miplevel, y + spec.y, yend,
+                                     z + spec.z, chbegin, chend, format,
                                      (char*)data + z * zstride + y * ystride,
                                      xstride, ystride);
                 if (progress_callback)
@@ -976,7 +953,7 @@ ImageInput::read_native_deep_image(int subimage, int miplevel,
         return false;
 
     if (spec.depth > 1) {
-        error(
+        errorf(
             "read_native_deep_image is not supported for volume (3D) images.");
         return false;
         // FIXME? - not implementing 3D deep images for now.  The only
