@@ -1,32 +1,6 @@
-/*
-  Copyright 2008 Larry Gritz and the other authors and contributors.
-  All Rights Reserved.
-
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are
-  met:
-  * Redistributions of source code must retain the above copyright
-    notice, this list of conditions and the following disclaimer.
-  * Redistributions in binary form must reproduce the above copyright
-    notice, this list of conditions and the following disclaimer in the
-    documentation and/or other materials provided with the distribution.
-  * Neither the name of the software's owners nor the names of its
-    contributors may be used to endorse or promote products derived from
-    this software without specific prior written permission.
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
-  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
-  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
-  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
-  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
-  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
-  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
-  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
-  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
-  (This is the Modified BSD License)
-*/
+// Copyright 2008-present Contributors to the OpenImageIO project.
+// SPDX-License-Identifier: BSD-3-Clause
+// https://github.com/OpenImageIO/oiio/blob/master/LICENSE.md
 
 
 #include <cmath>
@@ -181,6 +155,15 @@ private:
                       / m_spec.tile_height;
         return xtile + ytile * nxtiles + ztile * nxtiles * nytiles;
     }
+
+    // Move data to scratch area if not already there.
+    void* move_to_scratch(const void* data, size_t nbytes)
+    {
+        if (m_scratch.empty() || (const unsigned char*)data != m_scratch.data())
+            m_scratch.assign((const unsigned char*)data,
+                             (const unsigned char*)data + nbytes);
+        return m_scratch.data();
+    }
 };
 
 
@@ -206,7 +189,7 @@ tiff_imageio_library_version()
 }
 
 OIIO_EXPORT const char* tiff_output_extensions[]
-    = { "tiff", "tif", "tx", "env", "sm", "vsm", nullptr };
+    = { "tif", "tiff", "tx", "env", "sm", "vsm", nullptr };
 
 OIIO_PLUGIN_EXPORTS_END
 
@@ -324,12 +307,22 @@ TIFFOutput::supports(string_view feature) const
 #define ICC_PROFILE_ATTR "ICCProfile"
 
 
+// Do all elements of vector d have value v?
+template<typename T>
+inline bool
+allval(const std::vector<T>& d, T v = T(0))
+{
+    return std::all_of(d.begin(), d.end(), [&](const T& a) { return a == v; });
+}
+
+
+
 bool
 TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
                  OpenMode mode)
 {
     if (mode == AppendMIPLevel) {
-        error("%s does not support MIP levels", format_name());
+        errorf("%s does not support MIP levels", format_name());
         return false;
     }
 
@@ -338,22 +331,31 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
 
     // Check for things this format doesn't support
     if (m_spec.width < 1 || m_spec.height < 1) {
-        error("Image resolution must be at least 1x1, you asked for %d x %d",
-              m_spec.width, m_spec.height);
+        errorf("Image resolution must be at least 1x1, you asked for %d x %d",
+               m_spec.width, m_spec.height);
         return false;
     }
     if (m_spec.tile_width) {
         if (m_spec.tile_width % 16 != 0 || m_spec.tile_height % 16 != 0
             || m_spec.tile_height == 0) {
-            error("Tile size must be a multiple of 16, you asked for %d x %d",
-                  m_spec.tile_width, m_spec.tile_height);
+            errorf("Tile size must be a multiple of 16, you asked for %d x %d",
+                   m_spec.tile_width, m_spec.tile_height);
             return false;
         }
     }
     if (m_spec.depth < 1)
         m_spec.depth = 1;
+    if (m_spec.channelformats.size()) {
+        if (allval(m_spec.channelformats, m_spec.format))
+            m_spec.channelformats.clear();
+        else {
+            errorf("%s does not support per-channel data formats",
+                   format_name());
+            return false;
+        }
+    }
 
-        // Open the file
+    // Open the file
 #ifdef _WIN32
     std::wstring wname = Strutil::utf8_to_utf16(name);
     m_tif = TIFFOpenW(wname.c_str(), mode == AppendSubimage ? "a" : "w");
@@ -361,7 +363,7 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
     m_tif = TIFFOpen(name.c_str(), mode == AppendSubimage ? "a" : "w");
 #endif
     if (!m_tif) {
-        error("Can't open \"%s\" for output.", name.c_str());
+        errorf("Could not open \"%s\"", name);
         return false;
     }
 
@@ -401,7 +403,8 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
         sampformat      = SAMPLEFORMAT_INT;
         break;
     case TypeDesc::UINT8:
-        if (m_bitspersample != 2 && m_bitspersample != 4)
+        if (m_bitspersample != 2 && m_bitspersample != 4
+            && m_bitspersample != 6)
             m_bitspersample = 8;
         sampformat = SAMPLEFORMAT_UINT;
         break;
@@ -410,7 +413,8 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
         sampformat      = SAMPLEFORMAT_INT;
         break;
     case TypeDesc::UINT16:
-        if (m_bitspersample != 10 && m_bitspersample != 12)
+        if (m_bitspersample != 10 && m_bitspersample != 12
+            && m_bitspersample != 14)
             m_bitspersample = 16;
         sampformat = SAMPLEFORMAT_UINT;
         break;
@@ -419,8 +423,9 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
         sampformat      = SAMPLEFORMAT_INT;
         break;
     case TypeDesc::UINT32:
-        m_bitspersample = 32;
-        sampformat      = SAMPLEFORMAT_UINT;
+        if (m_bitspersample != 24)
+            m_bitspersample = 32;
+        sampformat = SAMPLEFORMAT_UINT;
         break;
     case TypeDesc::HALF:
         // Adobe extension, see http://chriscox.org/TIFFTN3d1.pdf
@@ -462,10 +467,14 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
     m_photometric = (m_spec.nchannels >= 3 ? PHOTOMETRIC_RGB
                                            : PHOTOMETRIC_MINISBLACK);
 
-    string_view comp = m_spec.get_string_attribute("Compression", "zip");
+    string_view comp;
+    int qual;
+    std::tie(comp, qual) = m_spec.decode_compression_metadata("zip");
+
     if (Strutil::iequals(comp, "jpeg")
         && (m_spec.format != TypeDesc::UINT8 || m_spec.nchannels != 3)) {
         comp = "zip";  // can't use JPEG for anything but 3xUINT8
+        qual = -1;
     }
     m_compression = tiff_compression_code(comp);
     TIFFSetField(m_tif, TIFFTAG_COMPRESSION, m_compression);
@@ -490,13 +499,16 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
         if (m_predictor != PREDICTOR_NONE)
             TIFFSetField(m_tif, TIFFTAG_PREDICTOR, m_predictor);
         if (m_compression == COMPRESSION_ADOBE_DEFLATE) {
-            int q = m_spec.get_int_attribute("tiff:zipquality", -1);
-            if (q >= 0)
-                TIFFSetField(m_tif, TIFFTAG_ZIPQUALITY, OIIO::clamp(q, 1, 9));
+            qual = m_spec.get_int_attribute("tiff:zipquality", qual);
+            if (qual > 0)
+                TIFFSetField(m_tif, TIFFTAG_ZIPQUALITY,
+                             OIIO::clamp(qual, 1, 9));
         }
     } else if (m_compression == COMPRESSION_JPEG) {
-        TIFFSetField(m_tif, TIFFTAG_JPEGQUALITY,
-                     m_spec.get_int_attribute("CompressionQuality", 95));
+        if (qual <= 0)
+            qual = 95;
+        qual = OIIO::clamp(qual, 1, 100);
+        TIFFSetField(m_tif, TIFFTAG_JPEGQUALITY, qual);
         m_rowsperstrip = 64;
         m_spec.attribute("tiff:RowsPerStrip", m_rowsperstrip);
         TIFFSetField(m_tif, TIFFTAG_ROWSPERSTRIP, m_rowsperstrip);
@@ -527,7 +539,7 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
             if (source_is_rgb(m_spec)) {
                 // Case: RGB -> CMYK, do the conversions per pixel
                 m_convert_rgb_to_cmyk = true;
-                m_outputchans         = 4;  // output 4, not 4 chans
+                m_outputchans         = 4;  // output 4, not 3 chans
                 TIFFSetField(m_tif, TIFFTAG_SAMPLESPERPIXEL, m_outputchans);
                 TIFFSetField(m_tif, TIFFTAG_INKSET, INKSET_CMYK);
             } else if (source_is_cmyk(m_spec)) {
@@ -853,13 +865,13 @@ TIFFOutput::write_exif_data()
 
     // First, finish writing the current directory
     if (!TIFFWriteDirectory(m_tif)) {
-        error("failed TIFFWriteDirectory()");
+        errorf("failed TIFFWriteDirectory()");
         return false;
     }
 
     // Create an Exif directory
     if (TIFFCreateEXIFDirectory(m_tif) != 0) {
-        error("failed TIFFCreateEXIFDirectory()");
+        errorf("failed TIFFCreateEXIFDirectory()");
         return false;
     }
 
@@ -896,7 +908,7 @@ TIFFOutput::write_exif_data()
     // Now write the directory of Exif data
     uint64 dir_offset = 0;
     if (!TIFFWriteCustomDirectory(m_tif, &dir_offset)) {
-        error("failed TIFFWriteCustomDirectory() of the Exif data");
+        errorf("failed TIFFWriteCustomDirectory() of the Exif data");
         return false;
     }
     // Go back to the first directory, and add the EXIFIFD pointer.
@@ -939,54 +951,6 @@ TIFFOutput::contig_to_separate(int n, int nchans, const char* contig,
 
 
 
-// Convert T data[0..nvals-1] in-place to BITS_TO per sample, *packed*.
-// The bit width of T is definitely wider than BITS_TO. T should be an
-// unsigned type.
-template<typename T, int BITS_TO>
-static void
-convert_pack_bits(T* data, int nvals)
-{
-    const int BITS_FROM = sizeof(T) * 8;
-    T* in               = data;
-    T* out              = in - 1;  // because we'll increment first time through
-
-    int bitstofill = 0;
-    // Invariant: the next value to convert is *in. We're going to write
-    // the result of the conversion starting at *out, which still has
-    // bitstofill bits left before moving on to the next slot.
-    for (int i = 0; i < nvals; ++i) {
-        // Grab the next value and convert it
-        T val = bit_range_convert<BITS_FROM, BITS_TO>(*in++);
-        // If we have no more bits to fill in the slot, move on to the
-        // next slot.
-        if (bitstofill == 0) {
-            ++out;
-            *out       = 0;          // move to next slot and clear its bits
-            bitstofill = BITS_FROM;  // all bits are for the taking
-        }
-        if (bitstofill >= BITS_TO) {
-            // we can fit the whole val in this slot
-            *out |= val << (bitstofill - BITS_TO);
-            bitstofill -= BITS_TO;
-            // printf ("\t\t%d bits left\n", bitstofill);
-        } else {
-            // not enough bits -- will need to split across slots
-            int bitsinnext = BITS_TO - bitstofill;
-            *out |= val >> bitsinnext;
-            val &= (1 << bitsinnext) - 1;  // mask out bits we saved
-            ++out;
-            *out = 0;  // move to next slot and clear its bits
-            *out |= val << (BITS_FROM - bitsinnext);
-            bitstofill = BITS_FROM - bitsinnext;
-        }
-    }
-    // Because we filled in a big-endian way, swap bytes if we need to
-    if (littleendian())
-        swap_endian(data, nvals);
-}
-
-
-
 template<typename T>
 static void
 rgb_to_cmyk(int n, const T* rgb, size_t rgb_stride, T* cmyk, size_t cmyk_stride)
@@ -1035,25 +999,18 @@ TIFFOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
                            stride_t xstride)
 {
     m_spec.auto_stride(xstride, format, spec().nchannels);
-    const void* origdata = data;
     data = to_native_scanline(format, data, xstride, m_scratch, m_dither, y, z);
 
     // Handle weird photometric/color spaces
     std::vector<unsigned char> cmyk;
     if (m_photometric == PHOTOMETRIC_SEPARATED && m_convert_rgb_to_cmyk)
         data = convert_to_cmyk(spec().width, data, cmyk);
+    size_t scanline_vals = spec().width * m_outputchans;
 
     // Handle weird bit depths
     if (spec().format.size() * 8 != m_bitspersample) {
-        // Move to scratch area if not already there
-        imagesize_t nbytes = spec().scanline_bytes();
-        int nvals          = spec().width * m_outputchans;
-        if (data == origdata) {
-            m_scratch.assign((unsigned char*)data,
-                             (unsigned char*)data + nbytes);
-            data = m_scratch.data();
-        }
-        fix_bitdepth(m_scratch.data(), nvals);
+        data = move_to_scratch(data, scanline_vals * spec().format.size());
+        fix_bitdepth(m_scratch.data(), scanline_vals);
     }
 
     y -= m_spec.y;
@@ -1065,7 +1022,7 @@ TIFFOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
         char* separate            = nullptr;
         imagesize_t separate_size = plane_bytes * m_outputchans;
         if (separate_size <= (1 << 16))
-            separate = ALLOCA(char, separate_size);        // <=64k ? stack
+            separate = OIIO_ALLOCA(char, separate_size);   // <=64k ? stack
         else {                                             // >64k ? heap
             separate_heap.reset(new char[separate_size]);  // will auto-free
             separate = separate_heap.get();
@@ -1078,8 +1035,8 @@ TIFFOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
                                   c)
                 < 0) {
                 std::string err = oiio_tiff_last_error();
-                error("TIFFWriteScanline failed writing line y=%d,z=%d (%s)", y,
-                      z, err.size() ? err.c_str() : "unknown error");
+                errorf("TIFFWriteScanline failed writing line y=%d,z=%d (%s)",
+                       y, z, err.size() ? err.c_str() : "unknown error");
                 return false;
             }
         }
@@ -1087,17 +1044,11 @@ TIFFOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
         // No contig->separate is necessary.  But we still use scratch
         // space since TIFFWriteScanline is destructive when
         // TIFFTAG_PREDICTOR is used.
-        if (data == origdata) {
-            imagesize_t scanline_bytes = m_spec.width * m_spec.format.size()
-                                         * m_outputchans;
-            m_scratch.assign((unsigned char*)data,
-                             (unsigned char*)data + scanline_bytes);
-            data = m_scratch.data();
-        }
+        data = move_to_scratch(data, scanline_vals * m_spec.format.size());
         if (TIFFWriteScanline(m_tif, (tdata_t)data, y) < 0) {
             std::string err = oiio_tiff_last_error();
-            error("TIFFWriteScanline failed writing line y=%d,z=%d (%s)", y, z,
-                  err.size() ? err.c_str() : "unknown error");
+            errorf("TIFFWriteScanline failed writing line y=%d,z=%d (%s)", y, z,
+                   err.size() ? err.c_str() : "unknown error");
             return false;
         }
     }
@@ -1177,7 +1128,7 @@ TIFFOutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
         && (m_spec.format == TypeUInt8 || m_spec.format == TypeUInt16)
         // only if we're threading and don't enter the thread pool recursively!
         && pool->size() > 1
-        && !pool->this_thread_is_in_pool()
+        && !pool->is_worker()
         // and not if the feature is turned off
         && m_spec.get_int_attribute("tiff:multithread",
                                     OIIO::get_int_attribute("tiff:multithread"));
@@ -1247,15 +1198,15 @@ TIFFOutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
         // it needs is not yet done.
         tasks.wait_for_task(stripidx);
         if (!ok) {
-            error("Compression error");
+            errorf("Compression error");
             return false;
         }
         if (TIFFWriteRawStrip(m_tif, stripnum, (tdata_t)cbuf,
                               tmsize_t(compressed_len[stripidx]))
             < 0) {
             std::string err = oiio_tiff_last_error();
-            error("TIFFWriteRawStrip failed writing line y=%d,z=%d: %s", y, z,
-                  err.size() ? err.c_str() : "unknown error");
+            errorf("TIFFWriteRawStrip failed writing line y=%d,z=%d: %s", y, z,
+                   err.size() ? err.c_str() : "unknown error");
             return false;
         }
     }
@@ -1272,7 +1223,6 @@ TIFFOutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
         m_checkpointItems = 0;
     }
 
-
     if (y < yend && origdata != data)
         memcpy((void*)data, origdata, (yend - y) * m_spec.scanline_bytes(true));
 
@@ -1299,9 +1249,9 @@ TIFFOutput::write_tile(int x, int y, int z, TypeDesc format, const void* data,
     x -= m_spec.x;  // Account for offset, so x,y are file relative, not
     y -= m_spec.y;  // image relative
     z -= m_spec.z;
-    const void* origdata = data;  // Stash original pointer
     data = to_native_tile(format, data, xstride, ystride, zstride, m_scratch,
                           m_dither, x, y, z);
+    size_t tile_vals = spec().tile_pixels() * m_outputchans;
 
     // Handle weird photometric/color spaces
     std::vector<unsigned char> cmyk;
@@ -1310,15 +1260,8 @@ TIFFOutput::write_tile(int x, int y, int z, TypeDesc format, const void* data,
 
     // Handle weird bit depths
     if (spec().format.size() * 8 != m_bitspersample) {
-        // Move to scratch area if not already there
-        imagesize_t nbytes = spec().scanline_bytes();
-        int nvals          = int(spec().tile_pixels()) * m_outputchans;
-        if (data == origdata) {
-            m_scratch.assign((unsigned char*)data,
-                             (unsigned char*)data + nbytes);
-            data = m_scratch.data();
-        }
-        fix_bitdepth(m_scratch.data(), nvals);
+        data = move_to_scratch(data, tile_vals * spec().format.size());
+        fix_bitdepth(m_scratch.data(), int(tile_vals));
     }
 
     if (m_planarconfig == PLANARCONFIG_SEPARATE && m_spec.nchannels > 1) {
@@ -1331,7 +1274,7 @@ TIFFOutput::write_tile(int x, int y, int z, TypeDesc format, const void* data,
         char* separate            = NULL;
         imagesize_t separate_size = plane_bytes * m_outputchans;
         if (separate_size <= (1 << 16))
-            separate = ALLOCA(char, separate_size);        // <=64k ? stack
+            separate = OIIO_ALLOCA(char, separate_size);   // <=64k ? stack
         else {                                             // >64k ? heap
             separate_heap.reset(new char[separate_size]);  // will auto-free
             separate = separate_heap.get();
@@ -1343,9 +1286,9 @@ TIFFOutput::write_tile(int x, int y, int z, TypeDesc format, const void* data,
                               z, c)
                 < 0) {
                 std::string err = oiio_tiff_last_error();
-                error("TIFFWriteTile failed writing tile x=%d,y=%d,z=%d (%s)",
-                      x + m_spec.x, y + m_spec.y, z + m_spec.z,
-                      err.size() ? err.c_str() : "unknown error");
+                errorf("TIFFWriteTile failed writing tile x=%d,y=%d,z=%d (%s)",
+                       x + m_spec.x, y + m_spec.y, z + m_spec.z,
+                       err.size() ? err.c_str() : "unknown error");
                 return false;
             }
         }
@@ -1353,18 +1296,12 @@ TIFFOutput::write_tile(int x, int y, int z, TypeDesc format, const void* data,
         // No contig->separate is necessary.  But we still use scratch
         // space since TIFFWriteTile is destructive when
         // TIFFTAG_PREDICTOR is used.
-        if (data == origdata) {
-            imagesize_t tile_bytes = m_spec.tile_pixels() * m_spec.format.size()
-                                     * m_outputchans;
-            m_scratch.assign((unsigned char*)data,
-                             (unsigned char*)data + tile_bytes);
-            data = m_scratch.data();
-        }
+        data = move_to_scratch(data, tile_vals * m_spec.format.size());
         if (TIFFWriteTile(m_tif, (tdata_t)data, x, y, z, 0) < 0) {
             std::string err = oiio_tiff_last_error();
-            error("TIFFWriteTile failed writing tile x=%d,y=%d,z=%d (%s)",
-                  x + m_spec.x, y + m_spec.y, z + m_spec.z,
-                  err.size() ? err.c_str() : "unknown error");
+            errorf("TIFFWriteTile failed writing tile x=%d,y=%d,z=%d (%s)",
+                   x + m_spec.x, y + m_spec.y, z + m_spec.z,
+                   err.size() ? err.c_str() : "unknown error");
             return false;
         }
     }
@@ -1424,7 +1361,7 @@ TIFFOutput::write_tiles(int xbegin, int xend, int ybegin, int yend, int zbegin,
         && (m_spec.format == TypeUInt8 || m_spec.format == TypeUInt16)
         // only if we're threading and don't enter the thread pool recursively!
         && pool->size() > 1
-        && !pool->this_thread_is_in_pool()
+        && !pool->is_worker()
         // and not if the feature is turned off
         && m_spec.get_int_attribute("tiff:multithread",
                                     OIIO::get_int_attribute("tiff:multithread"));
@@ -1522,14 +1459,14 @@ TIFFOutput::write_tiles(int xbegin, int xend, int ybegin, int yend, int zbegin,
                 tasks.wait_for_task(tileno);
                 char* cbuf = compressed_scratch.get() + tileno * cbound;
                 if (!ok) {
-                    error("Compression error");
+                    errorf("Compression error");
                     return false;
                 }
                 if (TIFFWriteRawTile(m_tif, uint32_t(tile_index(x, y, z)), cbuf,
                                      compressed_len[tileno])
                     < 0) {
                     std::string err = oiio_tiff_last_error();
-                    error(
+                    errorf(
                         "TIFFWriteRawTile failed writing tile %d (x=%d,y=%d,z=%d): %s",
                         tile_index(x, y, z), x, y, z,
                         err.size() ? err.c_str() : "unknown error");
@@ -1595,13 +1532,40 @@ TIFFOutput::fix_bitdepth(void* data, int nvals)
     ASSERT(spec().format.size() * 8 != m_bitspersample);
 
     if (spec().format == TypeDesc::UINT16 && m_bitspersample == 10) {
-        convert_pack_bits<unsigned short, 10>((unsigned short*)data, nvals);
+        unsigned short* v = (unsigned short*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<16, 10>(v[i]);
+        bit_pack(cspan<unsigned short>(v, v + nvals), v, 10);
     } else if (spec().format == TypeDesc::UINT16 && m_bitspersample == 12) {
-        convert_pack_bits<unsigned short, 12>((unsigned short*)data, nvals);
+        unsigned short* v = (unsigned short*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<16, 12>(v[i]);
+        bit_pack(cspan<unsigned short>(v, v + nvals), v, 12);
+    } else if (spec().format == TypeDesc::UINT16 && m_bitspersample == 14) {
+        unsigned short* v = (unsigned short*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<16, 14>(v[i]);
+        bit_pack(cspan<unsigned short>(v, v + nvals), v, 14);
     } else if (spec().format == TypeDesc::UINT8 && m_bitspersample == 4) {
-        convert_pack_bits<unsigned char, 4>((unsigned char*)data, nvals);
+        unsigned char* v = (unsigned char*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<8, 4>(v[i]);
+        bit_pack(cspan<unsigned char>(v, v + nvals), v, 4);
     } else if (spec().format == TypeDesc::UINT8 && m_bitspersample == 2) {
-        convert_pack_bits<unsigned char, 2>((unsigned char*)data, nvals);
+        unsigned char* v = (unsigned char*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<8, 2>(v[i]);
+        bit_pack(cspan<unsigned char>(v, v + nvals), v, 2);
+    } else if (spec().format == TypeDesc::UINT8 && m_bitspersample == 6) {
+        unsigned char* v = (unsigned char*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<8, 6>(v[i]);
+        bit_pack(cspan<unsigned char>(v, v + nvals), v, 6);
+    } else if (spec().format == TypeDesc::UINT32 && m_bitspersample == 24) {
+        unsigned int* v = (unsigned int*)data;
+        for (int i = 0; i < nvals; ++i)
+            v[i] = bit_range_convert<32, 24>(v[i]);
+        bit_pack(cspan<unsigned int>(v, v + nvals), v, 24);
     } else {
         ASSERT(0 && "unsupported bit conversion -- shouldn't reach here");
     }
